<?php

declare(strict_types=1);

/*
 * This file is part of the ************************ package.
 * _____________                           _______________
 *  ______/     \__  _____  ____  ______  / /_  _________
 *   ____/ __   / / / / _ \/ __`\/ / __ \/ __ \/ __ \___
 *    __/ / /  / /_/ /  __/ /  \  / /_/ / / / / /_/ /__
 *      \_\ \_/\____/\___/_/   / / .___/_/ /_/ .___/
 *         \_\                /_/_/         /_/
 *
 * The PHP Framework For Code Poem As Free As Wind. <Query Yet Simple>
 * (c) 2010-2018 http://queryphp.com All rights reserved.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Leevel\Bootstrap\Auth;

use Leevel\Auth;
use Leevel\Response;

/**
 * 退出登录.
 *
 * @author Xiangmin Liu <635750556@qq.com>
 *
 * @since 2017.09.09
 *
 * @version 1.0
 */
trait Logout
{
    use field;

    /**
     * 登录退出.
     *
     * @return \Leevel\Http\Response
     */
    public function logout()
    {
        return $this->displayLoginout();
    }

    /**
     * 登录退出.
     *
     * @return array|\Leevel\Http\Response
     */
    public function displayLoginout()
    {
        Auth::logout();

        if ($this->isAjaxRequest()) {
            return [
                'message' => $this->getLogoutMessage(),
            ];
        }

        return Response::redirect(
            $this->getLogoutRedirect()
        )->

        with('login_out', $this->getLogoutMessage());
    }

    /**
     * 获取退出登录消息.
     *
     * @return string
     */
    protected function getLogoutMessage()
    {
        return __('退出成功');
    }

    /**
     * 获取退出转向地址
     *
     * @return string
     */
    protected function getLogoutRedirect()
    {
        return property_exists($this, 'logoutRedirect') ?
            $this->logoutRedirect :
            'auth/index';
    }
}
